/* -*- indent-tabs-mode: t; tab-width: 4; c-basic-offset: 4; -*- */
/*
   gtkcmphash.h
   Projet GtkCmpHash

   (C) Copyright 2007 Olivier Delhomme
   e-mail : olivierdelhomme@gmail.com
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.           
*/
#ifndef _GTK_CMP_HASH_H_
#define _GTK_CMP_HASH_H_

#include <bzlib.h> 
#include <glade/glade.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gstdio.h>
#include <glib/gi18n.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h> 
#include <openssl/md5.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>
#include <lats/lats.h>
#include <openssl/hmac.h>

#ifdef HAVE_CONFIG_H
 #include "config.h"
#endif /* HAVE_CONFIG_H */

#define ProgName "gtkcmphash"
#define ProgVersion "0.0.1"
#define ProgDate "20.07.2007"
#define ProgAuthor "Olivier Delhomme"

/**
 *  Structure regroupant le hash (en binaire) ainsi que sa longueur 
 */
typedef struct
{
  guchar *hash;    /* l'criture en binaire du hash  */
  guint hash_len;  /* la longueur de cette dernire  */
} hex_mdp_t;


/**
 *  Structures de stockage des hash issus des diffrents fichiers
 *  composant le hashset
 *  file_hash_t : pour un seul fichier (utilis dans une liste pour viter les collisions)
 *  arbre_t : arbre pour grer l'ensemble des hash et faire tomber le
 *            nombre de comparaisons (n branches soit 16^n listes)
 *  hash_t : Super-structure permettant une simplification des appels
 *           de fonctions
 */
typedef struct
{
  gchar *hashset_name;  /* Le nom du hashset       */
  gchar *filename;      /* Le nom du fichier       */
  guchar *hash_md5;     /* le md5 du fichier       */
  guint  len_md5;       /* longueur du md5         */
  guchar *hash_sha1;    /* le sha1 du fichier      */
  guint  len_sha1;      /* longueur du sha1        */
  guchar *hash_ripemd;  /* le ripemd160 du fichier */
  guint  len_ripemd;    /* longueur du ripemd160   */
} file_hash_t;

typedef struct
{
  void *array[16]; /* un tableau d'indirections 0 -> 9 + a -> f
		      qui pointe soit sur un structure arbre_t soit sur une liste de compte_t      */
  guint niveau;    /* un indicateur du niveau dans lequel on se trouve (gnralement entre 1 et 3) */
} arbre_t;

typedef struct
{
  guint nb_niveau; /* Nombre de niveaux (1 si on n'a qu'un seul niveau d'indirection) */
  arbre_t *arbre;  /* l'arbre des hash  */
} hash_t;

/**
 *  Structure pour la barre de progression
 */
typedef struct
{
  guint64 max;               /* maximum value                                                        */
  guint64 value;             /* current value                                                        */
  guint64 max_file;          /* maximum value for the file progress bar                              */
  guint64 value_file;        /* current value for the file progress bar                              */
  GtkProgressBar *pb_global; /* la barre de progression principale                                   */
  GtkProgressBar *pb_file;   /* la barre de progression secondaire                                   */
  GtkLabel *pb_label;        /* le label situ juste au dessus de la barre de progression principale */
} p_bar_t;


/**
 *  Structure pour la gestion des options
 */
typedef struct
{
  gboolean include_dir;  /* indique si l'on doit inclure le rpertoire dans le nom du fichier dans file_hash_t */
} options_t;

typedef struct 
{
  GSList *found;      /* les hashs de la liste qui sont prsent dans le hashset */
  GSList *not_found;  /* les hashs de la liste qui ne sont PAS dans le hashset  */
} found_or_not_t;


/**
 *  Structure permettant de retourner le nombre d'octets
 *  non compresss traits ainsi que le nombre d'octets
 *  compresss que cela reprsente ainsi que le nombre
 *  de hashs
 */
typedef struct
{
  unsigned int uncompressed;
  unsigned int compressed;
  unsigned int nb_hash;
} bzip2_result_t;


/**
 *  Structure principale 
 */
typedef struct
{
  GladeXML *xml;          /* la dfinition XML de l'interface glade                               */
  GList *location_list;   /* la liste des localisation o l'on peut trouver les fichiers de conf  */
  log_t *log;             /* pour pouvoir logguer les messages                                    */
  p_bar_t *pb;            /* pour la gestion de la fentre de progression                         */
  GSList *file_hash_list; /* Liste des fichiers hash (d'un rpertoire)                           */
  options_t *opts;        /* Options ou prfrences                                               */
  hash_t *tronc;          /* Le tronc o l'on va stocker le hashset                               */
  found_or_not_t *dedans_ou_pas; /* dans le hashset ou pas.                                       */
} main_struct_t;

#include "log_window.h"
#include "file_io.h"
#include "hashing_struct.h"
#include "progress_window.h"
#include "pref_window.h"

#endif /* _GTK_CMP_HASH_H_ */
